/**
  ******************************************************************************
  * @file    tsensor.c
  * @author  MCD Application Team
  * @version V0.0.1RC1
  * @date    08/27/2009
  * @brief   This file includes driver for the Temperature Sensor STLM75M2E
  *          mounted on STM8L1526-EVAL board.
  ******************************************************************************
  * @copy
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2009 STMicroelectronics</center></h2>
  */


// Includes ------------------------------------------------------------------
#include "tsensor.h"

// Private typedef -----------------------------------------------------------
// Private define ------------------------------------------------------------
#define LM75_ADDR                  0x90 /* LM75 address */
#define LM75_SD_SET                0x01 /* Set SD bit in the configuration register */
#define LM75_SD_RESET              0xFE /* Reset SD bit in the configuration register */

#define LM75_I2C_SPEED             320000
#define I2C_LM75_SLAVE_ADDRESS7    0xA0
#define LM75_SMBUS_Speed           100000
#define SMBUS_LM75_SLAVE_ADDRESS7  0xA0

// Private macro -------------------------------------------------------------
// Private variables ---------------------------------------------------------
// Private function prototypes -----------------------------------------------
// Private functions ---------------------------------------------------------

/**
  * @brief  Initializes the I2C1.
  * @param  None
  * @retval None
  */
void I2C_LM75_Init(void)
{
  // Configure PC.4 as Input pull-up, used as TemperatureSensor_INT 
  GPIO_Init(LM75_I2C_GPIO, LM75_INT, GPIO_Mode_In_PU_IT);

  // I2C DeInit 
  I2C_DeInit(LM75_I2C);

  // I2C configuration 
  I2C_Init(LM75_I2C, LM75_I2C_SPEED, I2C_LM75_SLAVE_ADDRESS7,
           I2C_Mode_I2C, I2C_DutyCycle_16_9, I2C_Ack_Enable,
           I2C_AcknowledgedAddress_7bit);
}

/**
  * @brief  Initializes the I2C1 in SMBUS mode.
  * @param  None
  * @retval None
  */
void SMBUS_LM75_Init(void)
{
  // Configure PC.4 as Input pull-up, used as TemperatureSensor_INT 
  //GPIO_Init(LM75_I2C_GPIO, LM75_INT, GPIO_Mode_In_PU_No_IT);

  // I2C DeInit 
  I2C_DeInit(LM75_I2C);
  // I2C configuration in SMBUS mode
  I2C_Init(LM75_I2C, LM75_SMBUS_Speed, I2C_LM75_SLAVE_ADDRESS7,
           I2C_Mode_SMBusDevice, I2C_DutyCycle_2, I2C_Ack_Enable,
           I2C_AcknowledgedAddress_7bit);
  // Enables the I2C ARP 
  I2C_ARPCmd(LM75_I2C, ENABLE);
}

/**
  * @brief  Checks the LM75 status.
  * @param  None
  * @retval None
  */
ErrorStatus I2C_SMBUS_LM75_Status(void)
{
  uint32_t I2C_TimeOut = I2C_TIMEOUT;

  // Clear the I2C1 AF flag 
  I2C_ClearFlag(I2C1, I2C_FLAG_AF);

  // Enable I2C1 acknowledgement if it is already disabled by other function 
  I2C_AcknowledgeConfig(I2C1, ENABLE);

  // ---------------------------- Transmission Phase ---------------------------

  // Send I2C1 START condition 
  I2C_GenerateSTART(I2C1, ENABLE);

  // Test on I2C1 EV5 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT))  /* EV5 */
  {
  }

  // Send STLM75 slave address for write 
  I2C_Send7bitAddress(I2C1, LM75_ADDR, I2C_Direction_Transmitter);

  while ((!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED)) && I2C_TimeOut)/* EV6 */
  {
    I2C_TimeOut--;
  }

  if (I2C_GetFlagStatus(I2C1, I2C_FLAG_AF) != 0x00)
  {
    return ERROR;
  }
  else
  {
    return SUCCESS;
  }
}
/**
  * @brief  Read the specified register from the LM75.
  * @par Parameters:
  * RegValue: this member specifies the register to read:
  *                    - LM75_REG_TEMP: temperature register
  *                    - LM75_REG_TOS: Over-limit temperature register
  *                    - LM75_REG_THYS: Hysteresis temperature register
  * @retval None
  */
uint16_t I2C_SMBUS_LM75_Reg_Read(uint8_t RegName)
{
  __IO uint16_t RegValue = 0;

  // Enable I2C1 acknowledgement if it is already disabled by other function 
  I2C_AcknowledgeConfig(I2C1, ENABLE);

  // --------------------------- Transmission Phase ----------------------------
  // Send I2C1 START condition 
  I2C_GenerateSTART(I2C1, ENABLE);

  // Test on I2C1 EV5 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT))  /* EV5 */
  {
  }

  // Send STLM75 slave address for write 
  I2C_Send7bitAddress(I2C1, LM75_ADDR, I2C_Direction_Transmitter);

  // Test on I2C1 EV6 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED)) /* EV6 */
  {
  }

  // Send the specified register data pointer 
  I2C_SendData(I2C1, RegName);

  // Test on I2C1 EV8 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED)) /* EV8 */
  {
  }

  // ------------------------------ Reception Phase ----------------------------
  // Send Re-STRAT condition 
  I2C_GenerateSTART(I2C1, ENABLE);

  // Test on EV5 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT))  /* EV5 */
  {
  }

  // Send STLM75 slave address for read 
  I2C_Send7bitAddress(I2C1, LM75_ADDR, I2C_Direction_Receiver);

  // Test on EV6 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED))  /* EV6 */
  {
  }

  // Test on EV7 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_RECEIVED))  /* EV7 */
  {
  }

  // Store I2C1 received data 
  RegValue = (uint16_t)(I2C_ReceiveData(I2C1) << 8);

  // Disable I2C1 acknowledgement 
  I2C_AcknowledgeConfig(I2C1, DISABLE);

  // Send I2C1 STOP Condition 
  I2C_GenerateSTOP(I2C1, ENABLE);

  // Test on RXNE flag 
  while (I2C_GetFlagStatus(I2C1, I2C_FLAG_RXNE) == RESET);

  // Store I2C1 received data 
  RegValue |= I2C_ReceiveData(I2C1);

  // Return register value 
  return (RegValue);
}
/**
  * @brief  Write to the specified register of the LM75.
  * @par Parameters:
  * RegValue: this member specifies the register to write:
  *                    - LM75_REG_TOS: Over-limit temperature register
  *                    - LM75_REG_THYS: Hysteresis temperature register
  * @retval None
  */
void I2C_SMBUS_LM75_Reg_Write(uint8_t RegName, uint16_t RegValue)
{
  // -------------------------------- Transmission Phase -----------------------
  // Send I2C1 START condition 
  I2C_GenerateSTART(I2C1, ENABLE);

  // Test on I2C1 EV5 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT))  /* EV5 */
  {
  }

  // Send STLM75 slave address for write 
  I2C_Send7bitAddress(I2C1, LM75_ADDR, I2C_Direction_Transmitter);

  // Test on I2C1 EV6 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED)) /* EV6 */
  {
  }

  // Send the specified register data pointer 
  I2C_SendData(I2C1, RegName);

  // Test on I2C1 EV8 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED)) /* EV8 */
  {
  }

  // Send I2C1 data 
  I2C_SendData(I2C1, (uint8_t)(RegValue >> 8));

  // Test on I2C1 EV8 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED)) /* EV8 */
  {
  }

  // Send I2C1 data 
  I2C_SendData(I2C1, (uint8_t)RegValue);

  // Test on I2C1 EV8 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED)) /* EV8 */
  {
  }

  // Send I2C1 STOP Condition 
  I2C_GenerateSTOP(I2C1, ENABLE);
}
/**
  * @brief  Read Temperature register of LM75: double temperature value.
  * @param  None
  * @retval None
  */
uint16_t I2C_SMBUS_LM75_Temp_Read(void)
{
  __IO uint16_t RegValue = 0;

  // Enable I2C1 acknowledgement if it is already disabled by other function 
  I2C_AcknowledgeConfig(I2C1, ENABLE);

  // ------------------------------------- Transmission Phase ------------------
  // Send I2C1 START condition 
  I2C_GenerateSTART(I2C1, ENABLE);

  // Test on I2C1 EV5 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT))  /* EV5 */
  {
  }

  // Send STLM75 slave address for write 
  I2C_Send7bitAddress(I2C1, LM75_ADDR, I2C_Direction_Transmitter);

  // Test on I2C1 EV6 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED)) /* EV6 */
  {
  }

  // Send the temperature register data pointer 
  I2C_SendData(I2C1, LM75_REG_TEMP);

  // Test on I2C1 EV8 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED)) /* EV8 */
  {
  }

  // -------------------------------- Reception Phase --------------------------
  // Send Re-STRAT condition 
  I2C_GenerateSTART(I2C1, ENABLE);

  // Test on EV5 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT))  /* EV5 */
  {
  }

  // Send STLM75 slave address for read 
  I2C_Send7bitAddress(I2C1, LM75_ADDR, I2C_Direction_Receiver);

  // Test on EV6 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED))  /* EV6 */
  {
  }

  // Test on EV7 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_RECEIVED))  /* EV7 */
  {
  }

  // Store I2C1 received data 
  RegValue = I2C_ReceiveData(I2C1) << 8;

  // Disable I2C1 acknowledgement 
  I2C_AcknowledgeConfig(I2C1, DISABLE);

  // Send I2C1 STOP Condition 
  I2C_GenerateSTOP(I2C1, ENABLE);

  // Test on RXNE flag 
  while (I2C_GetFlagStatus(I2C1, I2C_FLAG_RXNE) == RESET);

  // Store I2C1 received data 
  RegValue |= I2C_ReceiveData(I2C1);

  // Return Temperature value 
  return (RegValue >> 7);
}
/**
  * @brief  Read the configuration register from the LM75.
  * @param  None
  * @retval None
  */
uint8_t I2C_SMBUS_LM75_ConfReg_Read(void)
{
  __IO uint8_t RegValue = 0;

  // Enable I2C1 acknowledgement if it is already disabled by other function 
  I2C_AcknowledgeConfig(I2C1, ENABLE);
  // ----------------------------- Transmission Phase --------------------------
  // Send I2C1 START condition 
  I2C_GenerateSTART(I2C1, ENABLE);

  // Test on I2C1 EV5 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT))  /* EV5 */
  {
  }

  // Send STLM75 slave address for write 
  I2C_Send7bitAddress(I2C1, LM75_ADDR, I2C_Direction_Transmitter);

  // Test on I2C1 EV6 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED)) /* EV6 */
  {
  }

  // Send the configuration register data pointer 
  I2C_SendData(I2C1, LM75_REG_CONF);

  // Test on I2C1 EV8 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED)) /* EV8 */
  {
  }

  // ----------------------------- Reception Phase -----------------------------
  // Send Re-STRAT condition 
  I2C_GenerateSTART(I2C1, ENABLE);

  // Test on EV5 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT))  /* EV5 */
  {
  }

  // Send STLM75 slave address for read 
  I2C_Send7bitAddress(I2C1, LM75_ADDR, I2C_Direction_Receiver);

  // Test on EV6 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED))  /* EV6 */
  {
  }

  // Disable I2C1 acknowledgement 
  I2C_AcknowledgeConfig(I2C1, DISABLE);

  // Send I2C1 STOP Condition 
  I2C_GenerateSTOP(I2C1, ENABLE);

  // Test on RXNE flag 
  while (I2C_GetFlagStatus(I2C1, I2C_FLAG_RXNE) == RESET);

  // Store I2C1 received data 
  RegValue = I2C_ReceiveData(I2C1);

  // Return configuration register value 
  return (RegValue);
}
/**
  * @brief  Write to the configuration register of the LM75.
  * @param  None
  * @retval None
  */
void I2C_SMBUS_LM75_ConfReg_Write(uint8_t RegValue)
{
  // -------------------------------- Transmission Phase -----------------------
  // Send I2C1 START condition 
  I2C_GenerateSTART(I2C1, ENABLE);

  // Test on I2C1 EV5 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT))  /* EV5 */
  {
  }

  // Send STLM75 slave address for write 
  I2C_Send7bitAddress(I2C1, LM75_ADDR, I2C_Direction_Transmitter);

  // Test on I2C1 EV6 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED)) /* EV6 */
  {
  }

  // Send the configuration register data pointer 
  I2C_SendData(I2C1, LM75_REG_CONF);

  // Test on I2C1 EV8 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED)) /* EV8 */
  {
  }

  // Send I2C1 data 
  I2C_SendData(I2C1, RegValue);

  // Test on I2C1 EV8 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED)) /* EV8 */
  {
  }

  // Send I2C1 STOP Condition 
  I2C_GenerateSTOP(I2C1, ENABLE);
}
/**
  * @brief  Enables or disables the LM75.
  * @param  None
  * @retval None
  */
void I2C_SMBUS_LM75_ShutDown(FunctionalState NewState)
{
  __IO uint8_t RegValue = 0;

  // ---------------------------- Transmission Phase ---------------------------
  // Send I2C1 START condition 
  I2C_GenerateSTART(I2C1, ENABLE);

  // Test on I2C1 EV5 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT))  /* EV5 */
  {
  }

  // Send STLM75 slave address for write 
  I2C_Send7bitAddress(I2C1, LM75_ADDR, I2C_Direction_Transmitter);

  // Test on I2C1 EV6 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED)) /* EV6 */
  {
  }

  // Send the configuration register data pointer 
  I2C_SendData(I2C1, LM75_REG_CONF);

  // Test on I2C1 EV8 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED))  /* EV8 */
  {
  }

  // -------------------------------- Reception Phase --------------------------
  // Send Re-STRAT condition 
  I2C_GenerateSTART(I2C1, ENABLE);

  // Test on EV5 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT))  /* EV5 */
  {
  }

  // Send STLM75 slave address for read 
  I2C_Send7bitAddress(I2C1, LM75_ADDR, I2C_Direction_Receiver);

  // Test on EV6 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED))  /* EV6 */
  {
  }

  // Disable I2C1 acknowledgement 
  I2C_AcknowledgeConfig(I2C1, DISABLE);

  // Send I2C1 STOP Condition 
  I2C_GenerateSTOP(I2C1, ENABLE);

  // Test on RXNE flag 
  while (I2C_GetFlagStatus(I2C1, I2C_FLAG_RXNE) == RESET);

  // Store I2C1 received data 
  RegValue = I2C_ReceiveData(I2C1);

  // ------------------------------------ Transmission Phase -------------------
  // Send I2C1 START condition 
  I2C_GenerateSTART(I2C1, ENABLE);

  // Test on I2C1 EV5 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT))  /* EV5 */
  {
  }

  // Send STLM75 slave address for write 
  I2C_Send7bitAddress(I2C1, LM75_ADDR, I2C_Direction_Transmitter);

  // Test on I2C1 EV6 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED)) /* EV6 */
  {
  }

  // Send the configuration register data pointer 
  I2C_SendData(I2C1, LM75_REG_CONF);

  // Test on I2C1 EV8 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED)) /* EV8 */
  {
  }

  // Enable or disable SD bit 
  if (NewState != DISABLE)
  {
    // Enable LM75 
    I2C_SendData(I2C1, RegValue & LM75_SD_RESET);
  }
  else
  {
    // Disable LM75 
    I2C_SendData(I2C1, RegValue | LM75_SD_SET);
  }

  // Test on I2C1 EV8 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED)) /* EV8 */
  {
  }

  // Send I2C1 STOP Condition 
  I2C_GenerateSTOP(I2C1, ENABLE);

}
/**
  * @brief  Configures the alarm methode for the temperature sensor STLM75
  * @param  Alarm_Mode Alarm mode
 *  This parameter can be one of the @ref Alarm_Mode_TypeDef enumeration.
  * @param Alarm_FaultTolerance  Alarm fault tolerance
  *  This parameter can be one of the @ref Alarm_FaultTolerance_TypeDef enumeration.
  * @retval None
  */
void I2C_SMBUS_AlarmConfig(Alarm_Mode_TypeDef Alarm_Mode, Alarm_FaultTolerance_TypeDef Alarm_FaultTolerance)
{
  __IO uint8_t RegValue = 0;

  // ---------------------------- Transmission Phase ---------------------------
  // Send I2C1 START condition 
  I2C_GenerateSTART(I2C1, ENABLE);

  // Test on I2C1 EV5 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT))  /* EV5 */
  {
  }

  // Send STLM75 slave address for write 
  I2C_Send7bitAddress(I2C1, LM75_ADDR, I2C_Direction_Transmitter);

  // Test on I2C1 EV6 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED)) /* EV6 */
  {
  }

  // Send the configuration register data pointer 
  I2C_SendData(I2C1, LM75_REG_CONF);

  // Test on I2C1 EV8 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED))  /* EV8 */
  {
  }

  // -------------------------------- Reception Phase --------------------------
  // Send Re-STRAT condition 
  I2C_GenerateSTART(I2C1, ENABLE);

  // Test on EV5 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT))  /* EV5 */
  {
  }

  // Send STLM75 slave address for read 
  I2C_Send7bitAddress(I2C1, LM75_ADDR, I2C_Direction_Receiver);

  // Test on EV6 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED))  /* EV6 */
  {
  }

  // Disable I2C1 acknowledgement 
  I2C_AcknowledgeConfig(I2C1, DISABLE);

  // Send I2C1 STOP Condition 
  I2C_GenerateSTOP(I2C1, ENABLE);

  // Test on RXNE flag 
  while (I2C_GetFlagStatus(I2C1, I2C_FLAG_RXNE) == RESET);

  // Store I2C1 received data 
  RegValue = I2C_ReceiveData(I2C1);

  // ------------------------------------ Transmission Phase -------------------
  // Send I2C1 START condition 
  I2C_GenerateSTART(I2C1, ENABLE);

  // Test on I2C1 EV5 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT))  /* EV5 */
  {
  }

  // Send STLM75 slave address for write 
  I2C_Send7bitAddress(I2C1, LM75_ADDR, I2C_Direction_Transmitter);

  // Test on I2C1 EV6 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED)) /* EV6 */
  {
  }

  // Send the configuration register data pointer 
  I2C_SendData(I2C1, LM75_REG_CONF);

  // Test on I2C1 EV8 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED)) /* EV8 */
  {
  }

  // Send Configue 
  I2C_SendData(I2C1, (RegValue & (Alarm_Mode | Alarm_FaultTolerance)));

  // Test on I2C1 EV8 and clear it 
  while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED)) /* EV8 */
  {
  }

  // Send I2C1 STOP Condition 
  I2C_GenerateSTOP(I2C1, ENABLE);
}
