/**
  ******************************************************************************
  * @file    i2c_ee.c
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    09/14/2009
  * @brief   This file includes driver for the M24C64 I2C EEPROM mounted on
  *          STM8L1526-EVAL board.
  ******************************************************************************
  * @copy
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2009 STMicroelectronics</center></h2>
  */

// Includes ------------------------------------------------------------------
#include "i2c_ee.h"


 // @addtogroup Utilities

 // @addtogroup I2C_EEPROM

 // @defgroup I2C_EEPROM_Types


 // @defgroup I2C_EEPROM_Defines
#define EE_I2C_SPEED             300000
#define EE_I2C_SLAVE_ADDRESS7    0xA0
#define EE_PAGESIZE              32



 // @defgroup I2C_EEPROM_Macros


 // @defgroup I2C_EEPROM_Variables
uint16_t EepromAddress;
 // @defgroup I2C_EEPROM_Prototypes
void delay(__IO uint16_t nCount);

 // @defgroup I2C_EEPROM_Functions

/**
  * @brief  Initializes peripherals used by the I2C EEPROM driver.
  * @param  None
  * @retval None
  */
void I2C_EE_Init()
{
  I2C_DeInit(EE_I2C);
  // I2C configuration 
  I2C_Init(EE_I2C, EE_I2C_SPEED, EE_I2C_SLAVE_ADDRESS7, I2C_Mode_I2C, I2C_DutyCycle_16_9,
           I2C_Ack_Enable, I2C_AcknowledgedAddress_7bit);

  // Select the EEPROM address according to the state of E0, E1, E2 pins 
  EepromAddress = EEPROM_HW_ADDRESS;
  delay(0xFFFF);
}

/**
  * @brief  Writes one byte to the I2C EEPROM.
  * @param  pBuffer : pointer to the buffer  containing the data to be
  *   written to the EEPROM.
  * @param  WriteAddr : EEPROM's internal address to write to.
  * @retval None
  */
void I2C_EE_ByteWrite(uint8_t* pBuffer, uint16_t WriteAddr)
{
  // Send STRAT condition 
  I2C_GenerateSTART(EE_I2C, ENABLE);

  // Test on EV5 and clear it 
  while (!I2C_CheckEvent(EE_I2C, I2C_EVENT_MASTER_MODE_SELECT));

  // Send EEPROM address for write 
  I2C_Send7bitAddress(EE_I2C, EepromAddress, I2C_Direction_Transmitter);

  // Test on EV6 and clear it 
  while (!I2C_CheckEvent(EE_I2C, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));


  // Send the EEPROM's internal address to write to : MSB of the address first 
  I2C_SendData(EE_I2C, (uint8_t)(WriteAddr >> 8));

  // Test on EV8 and clear it 
  while (!I2C_CheckEvent(EE_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED));

  // Send the EEPROM's internal address to write to : LSB of the address 
  I2C_SendData(EE_I2C, (uint8_t)WriteAddr);

  // Test on EV8 and clear it 
  while (!I2C_CheckEvent(EE_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED));

  // Send the byte to be written 
  I2C_SendData(EE_I2C, *pBuffer);

  // Test on EV8 and clear it 
  while (!I2C_CheckEvent(EE_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED));

  // Send STOP condition 
  I2C_GenerateSTOP(EE_I2C, ENABLE);
}

/**
  * @brief  Reads a block of data from the EEPROM.
  * @param  pBuffer : pointer to the buffer that receives the data read
  *   from the EEPROM.
  * @param  ReadAddr : EEPROM's internal address to read from.
  * @param  NumByteToRead : number of bytes to read from the EEPROM.
  * @retval None
  */
void I2C_EE_BufferRead(uint8_t* pBuffer, uint16_t ReadAddr, uint16_t NumByteToRead)
{
  while (I2C_GetFlagStatus(EE_I2C, I2C_FLAG_BUSY));

  // Send START condition 
  I2C_GenerateSTART(EE_I2C, ENABLE);

  // Test on EV5 and clear it 
  while (!I2C_CheckEvent(EE_I2C, I2C_EVENT_MASTER_MODE_SELECT));

  // Send EEPROM address for write 
  I2C_Send7bitAddress(EE_I2C, EepromAddress, I2C_Direction_Transmitter);

  // Test on EV6 and clear it 
  while (!I2C_CheckEvent(EE_I2C, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));
  I2C_GetFlagStatus(I2C1, I2C_FLAG_ADDR);

  // Send the EEPROM's internal address to read from: MSB of the address first 
  I2C_SendData(EE_I2C, (uint8_t)((ReadAddr & 0xFF00) >> 8));

  // Test on EV8 and clear it 
  while (!I2C_CheckEvent(EE_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED));

  // Send the EEPROM's internal address to read from: LSB of the address 
  I2C_SendData(EE_I2C, (uint8_t)(ReadAddr & 0x00FF));


  // Test on EV8 and clear it 
  while (!I2C_CheckEvent(EE_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED));

  // Send STRAT condition a second time 
  I2C_GenerateSTART(EE_I2C, ENABLE);

  // Test on EV5 and clear it 
  while (!I2C_CheckEvent(EE_I2C, I2C_EVENT_MASTER_MODE_SELECT));

  // Send EEPROM address for read 
  I2C_Send7bitAddress(EE_I2C, EepromAddress, I2C_Direction_Receiver);

  // Test on EV6 and clear it 
  while (!I2C_CheckEvent(EE_I2C, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED));

  // While there is data to be read 
  while (NumByteToRead)
  {
    if (NumByteToRead == 1)
    {
      // Disable Acknowledgement 
      I2C_AcknowledgeConfig(EE_I2C, DISABLE);

      // Send STOP Condition 
      I2C_GenerateSTOP(EE_I2C, ENABLE);


      while (I2C_GetFlagStatus(EE_I2C, I2C_FLAG_RXNE) == RESET);
      // Read a byte from the EEPROM 
      *pBuffer = I2C_ReceiveData(EE_I2C);

      // Point to the next location where the byte read will be saved 
      pBuffer++;

      // Decrement the read bytes counter 
      NumByteToRead--;
    }

    // Test on EV7 and clear it 
    if (I2C_CheckEvent(EE_I2C, I2C_EVENT_MASTER_BYTE_RECEIVED))
    {
      // Read a byte from the EEPROM 
      *pBuffer = I2C_ReceiveData(EE_I2C);

      // Point to the next location where the byte read will be saved 
      pBuffer++;

      // Decrement the read bytes counter 
      NumByteToRead--;
    }
  }

  // Enable Acknowledgement to be ready for another reception 
  I2C_AcknowledgeConfig(EE_I2C, ENABLE);
}

/**
  * @brief  Writes buffer of data to the I2C EEPROM.
  * @param  pBuffer : pointer to the buffer  containing the data to be
  *   written to the EEPROM.
  * @param  WriteAddr : EEPROM's internal address to write to.
  * @param  NumByteToWrite : number of bytes to write to the EEPROM.
  * @retval None
  */
void I2C_EE_BufferWrite(uint8_t* pBuffer, uint16_t WriteAddr, uint16_t NumByteToWrite)
{
  uint8_t NumOfPage = 0, NumOfSingle = 0, count = 0;

  count = EE_PAGESIZE - (WriteAddr % EE_PAGESIZE);
  NumOfPage =  NumByteToWrite / EE_PAGESIZE;
  NumOfSingle = NumByteToWrite % EE_PAGESIZE;

  // If WriteAddr is EE_PAGESIZE aligned  
  if ((WriteAddr % EE_PAGESIZE) == 0)
  {
    // If NumByteToWrite < EE_PAGESIZE 
    if (NumOfPage == 0)
    {
      I2C_EE_PageWrite(pBuffer, WriteAddr, NumOfSingle);
      I2C_EE_WaitEepromStandbyState();
    }
    // If NumByteToWrite > EE_PAGESIZE 
    else
    {
      while (NumOfPage--)
      {
        I2C_EE_PageWrite(pBuffer, WriteAddr, EE_PAGESIZE);
        I2C_EE_WaitEepromStandbyState();
        WriteAddr +=  EE_PAGESIZE;
        pBuffer += EE_PAGESIZE;
      }

      if (NumOfSingle != 0)
      {
        I2C_EE_PageWrite(pBuffer, WriteAddr, NumOfSingle);
        I2C_EE_WaitEepromStandbyState();
      }
    }
  }
  // If WriteAddr is not EE_PAGESIZE aligned  
  else
  {
    // If NumByteToWrite < EE_PAGESIZE 
    if (NumOfPage == 0)
    {
      /* If the number of data to be written is more than the remaining space
      in the current page: */
      if (NumByteToWrite > count)
      {
        // Write the data conained in same page 
        I2C_EE_PageWrite(pBuffer, WriteAddr, count);
        I2C_EE_WaitEepromStandbyState();

        // Write the remaining data in the following page 
        I2C_EE_PageWrite((uint8_t*)(pBuffer + count), (WriteAddr + count), (NumByteToWrite - count));
        I2C_EE_WaitEepromStandbyState();
      }
      else
      {
        I2C_EE_PageWrite(pBuffer, WriteAddr, NumOfSingle);
        I2C_EE_WaitEepromStandbyState();
      }
    }
    // If NumByteToWrite > EE_PAGESIZE 
    else
    {
      NumByteToWrite -= count;
      NumOfPage =  NumByteToWrite / EE_PAGESIZE;
      NumOfSingle = NumByteToWrite % EE_PAGESIZE;

      if (count != 0)
      {
        I2C_EE_PageWrite(pBuffer, WriteAddr, count);
        I2C_EE_WaitEepromStandbyState();
        WriteAddr += count;
        pBuffer += count;
      }

      while (NumOfPage--)
      {
        I2C_EE_PageWrite(pBuffer, WriteAddr, EE_PAGESIZE);
        I2C_EE_WaitEepromStandbyState();
        WriteAddr +=  EE_PAGESIZE;
        pBuffer += EE_PAGESIZE;
      }
      if (NumOfSingle != 0)
      {
        I2C_EE_PageWrite(pBuffer, WriteAddr, NumOfSingle);
        I2C_EE_WaitEepromStandbyState();
      }
    }
  }
}

/**
  * @brief  Writes more than one byte to the EEPROM with a single WRITE cycle.
  * @note   The number of byte can't exceed the EEPROM page size.
  * @param  pBuffer : pointer to the buffer containing the data to be
  *   written to the EEPROM.
  * @param  WriteAddr : EEPROM's internal address to write to.
  * @param  NumByteToWrite : number of bytes to write to the EEPROM.
  * @retval None
  */
void I2C_EE_PageWrite(uint8_t* pBuffer, uint16_t WriteAddr, uint8_t NumByteToWrite)
{
  // While the bus is busy 
  while (I2C_GetFlagStatus(EE_I2C, I2C_FLAG_BUSY));

  // Send START condition 
  I2C_GenerateSTART(EE_I2C, ENABLE);

  // Test on EV5 and clear it 
  while (!I2C_CheckEvent(EE_I2C, I2C_EVENT_MASTER_MODE_SELECT));

  // Send EEPROM address for write 
  I2C_Send7bitAddress(EE_I2C, EepromAddress, I2C_Direction_Transmitter);

  // Test on EV6 and clear it 
  while (!I2C_CheckEvent(EE_I2C, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));


  // Send the EEPROM's internal address to write to : MSB of the address first 
  I2C_SendData(EE_I2C, (uint8_t)((WriteAddr & 0xFF00) >> 8));

  // Test on EV8 and clear it 
  while (!I2C_CheckEvent(EE_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED));

  // Send the EEPROM's internal address to write to : LSB of the address 
  I2C_SendData(EE_I2C, (uint8_t)(WriteAddr & 0x00FF));

  // Test on EV8 and clear it 
  while (! I2C_CheckEvent(EE_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED));

  // While there is data to be written 
  while (NumByteToWrite--)
  {
    // Send the current byte 
    I2C_SendData(EE_I2C, *pBuffer);

    // Point to the next byte to be written 
    pBuffer++;

    // Test on EV8 and clear it 
    while (!I2C_CheckEvent(EE_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED));
  }

  // Send STOP condition 
  I2C_GenerateSTOP(EE_I2C, ENABLE);
}

/**
  * @brief  Wait for EEPROM Standby state
  * @param  None
  * @retval None
  */
void I2C_EE_WaitEepromStandbyState(void)
{
  __IO uint8_t SR1_Tmp = 0;

  do
  {
    // Send START condition 
    I2C_GenerateSTART(EE_I2C, ENABLE);

    delay(10);
    // Read EE_I2C SR1 register to clear pending flags 
    SR1_Tmp = I2C_ReadRegister(EE_I2C, I2C_Register_SR1);

    // Send EEPROM address for write 
    I2C_Send7bitAddress(EE_I2C, EepromAddress, I2C_Direction_Transmitter);

  }
  while (!(I2C_ReadRegister(EE_I2C, I2C_Register_SR1) & 0x0002));

  // Clear AF flag 
  I2C_ClearFlag(EE_I2C, I2C_FLAG_AF);

  // STOP condition 
  I2C_GenerateSTOP(EE_I2C, ENABLE);
}
/**
  * @brief Delay.
  * @param nCount
  * @retval None
  */
void delay(__IO uint16_t nCount)
{
  // Decrement nCount value 
  while (nCount != 0)
  {
    nCount--;
  }
}




