/**
  ******************************************************************************
  * @file    tsensor.c
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    09/14/2009
  * @brief   This file includes driver for the Temperature Sensor STLM75M2E
  *          mounted on STM8L1526-EVAL board.
  ******************************************************************************
  * @copy
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2009 STMicroelectronics</center></h2>
  */


// Includes ------------------------------------------------------------------
#include "tsensor.h"

// Private typedef -----------------------------------------------------------
// Private define ------------------------------------------------------------
#define LM75_ADDR                  0x90 /* LM75 address */
#define LM75_SD_SET                0x01 /* Set SD bit in the configuration register */
#define LM75_SD_RESET              0xFE /* Reset SD bit in the configuration register */

#define LM75_I2C_SPEED             320000
#define LM75_I2C_SLAVE_ADDRESS7    0xA0

// Private macro -------------------------------------------------------------
// Private variables ---------------------------------------------------------
// Private function prototypes -----------------------------------------------
// Private functions ---------------------------------------------------------

/**
  * @brief  Initializes the LM75_I2C.
  * @param  None
  * @retval None
  */
void I2C_LM75_Init(void)
{

  // Configure PC.4 as Input pull-up, used as TemperatureSensor_INT 
  GPIO_Init(LM75_I2C_GPIO, LM75_INT, GPIO_Mode_In_PU_IT);

  // I2C DeInit 
  I2C_DeInit(LM75_I2C);
  // I2C configuration 
  I2C_Init(LM75_I2C, LM75_I2C_SPEED, LM75_I2C_SLAVE_ADDRESS7, I2C_Mode_I2C, I2C_DutyCycle_16_9,
           I2C_Ack_Enable, I2C_AcknowledgedAddress_7bit);
}
/**
  * @brief  Checks the LM75 status.
  * @param  None
  * @retval None
  */
ErrorStatus I2C_LM75_Status(void)
{
  uint32_t I2C_TimeOut = I2C_TIMEOUT;

  // Clear the LM75_I2C AF flag 
  I2C_ClearFlag(LM75_I2C, I2C_FLAG_AF);

  // Enable LM75_I2C acknowledgement if it is already disabled by other function 
  I2C_AcknowledgeConfig(LM75_I2C, ENABLE);

  // ---------------------------- Transmission Phase ---------------------------

  // Send LM75_I2C START condition 
  I2C_GenerateSTART(LM75_I2C, ENABLE);

  // Test on LM75_I2C EV5 and clear it 
  while ((!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_MODE_SELECT)) && I2C_TimeOut)  /* EV5 */
  {
    I2C_TimeOut--;
  }
  if (I2C_TimeOut == 0)
  {
    return ERROR;
  }
  I2C_TimeOut = I2C_TIMEOUT;
  // Send STLM75 slave address for write 
  I2C_Send7bitAddress(LM75_I2C, LM75_ADDR, I2C_Direction_Transmitter);
  while ((!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED)) && I2C_TimeOut)/* EV6 */
  {
    I2C_TimeOut--;
  }

  if ((I2C_GetFlagStatus(LM75_I2C, I2C_FLAG_AF) != 0x00) || (I2C_TimeOut == 0))
  {
    return ERROR;
  }
  else
  {
    return SUCCESS;
  }
}
/**
  * @brief  Read the specified register from the LM75.
  * @par Parameters:
  * RegValue: this member specifies the register to read:
  *                    - LM75_REG_TEMP: temperature register
  *                    - LM75_REG_TOS: Over-limit temperature register
  *                    - LM75_REG_THYS: Hysteresis temperature register
  * @retval None
  */
uint16_t I2C_LM75_Reg_Read(uint8_t RegName)
{
  __IO uint16_t RegValue = 0;

  // Enable LM75_I2C acknowledgement if it is already disabled by other function 
  I2C_AcknowledgeConfig(LM75_I2C, ENABLE);

  // --------------------------- Transmission Phase ----------------------------
  // Send LM75_I2C START condition 
  I2C_GenerateSTART(LM75_I2C, ENABLE);

  // Test on LM75_I2C EV5 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_MODE_SELECT))  /* EV5 */
  {
  }

  // Send STLM75 slave address for write 
  I2C_Send7bitAddress(LM75_I2C, LM75_ADDR, I2C_Direction_Transmitter);

  // Test on LM75_I2C EV6 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED)) /* EV6 */
  {
  }

  // Send the specified register data pointer 
  I2C_SendData(LM75_I2C, RegName);

  // Test on LM75_I2C EV8 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED)) /* EV8 */
  {
  }

  // ------------------------------ Reception Phase ----------------------------
  // Send Re-STRAT condition 
  I2C_GenerateSTART(LM75_I2C, ENABLE);

  // Test on EV5 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_MODE_SELECT))  /* EV5 */
  {
  }

  // Send STLM75 slave address for read 
  I2C_Send7bitAddress(LM75_I2C, LM75_ADDR, I2C_Direction_Receiver);

  // Test on EV6 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED))  /* EV6 */
  {
  }

  // Test on EV7 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_BYTE_RECEIVED))  /* EV7 */
  {
  }

  // Store LM75_I2C received data 
  RegValue = (uint16_t)(I2C_ReceiveData(LM75_I2C) << 8);

  // Disable LM75_I2C acknowledgement 
  I2C_AcknowledgeConfig(LM75_I2C, DISABLE);

  // Send LM75_I2C STOP Condition 
  I2C_GenerateSTOP(LM75_I2C, ENABLE);

  // Test on RXNE flag 
  while (I2C_GetFlagStatus(LM75_I2C, I2C_FLAG_RXNE) == RESET);

  // Store LM75_I2C received data 
  RegValue |= I2C_ReceiveData(LM75_I2C);

  // Return register value 
  return (RegValue);
}

/**
  * @brief  Write to the specified register of the LM75.
  * @par Parameters:
  * RegValue: this member specifies the register to write:
  *                    - LM75_REG_TOS: Over-limit temperature register
  *                    - LM75_REG_THYS: Hysteresis temperature register
  * @retval None
  */
void I2C_LM75_Reg_Write(uint8_t RegName, uint16_t RegValue)
{
  // -------------------------------- Transmission Phase -----------------------
  // Send LM75_I2C START condition 
  I2C_GenerateSTART(LM75_I2C, ENABLE);

  // Test on LM75_I2C EV5 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_MODE_SELECT))  /* EV5 */
  {
  }

  // Send STLM75 slave address for write 
  I2C_Send7bitAddress(LM75_I2C, LM75_ADDR, I2C_Direction_Transmitter);

  // Test on LM75_I2C EV6 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED)) /* EV6 */
  {
  }

  // Send the specified register data pointer 
  I2C_SendData(LM75_I2C, RegName);

  // Test on LM75_I2C EV8 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED)) /* EV8 */
  {
  }

  // Send LM75_I2C data 
  I2C_SendData(LM75_I2C, (uint8_t)(RegValue >> 8));

  // Test on LM75_I2C EV8 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED)) /* EV8 */
  {
  }

  // Send LM75_I2C data 
  I2C_SendData(LM75_I2C, (uint8_t)RegValue);

  // Test on LM75_I2C EV8 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED)) /* EV8 */
  {
  }

  // Send LM75_I2C STOP Condition 
  I2C_GenerateSTOP(LM75_I2C, ENABLE);
}

/**
  * @brief  Read Temperature register of LM75: double temperature value.
  * @param  None
  * @retval None
  */
uint16_t I2C_LM75_Temp_Read(void)
{
  __IO uint16_t RegValue = 0;

  // Enable LM75_I2C acknowledgement if it is already disabled by other function 
  I2C_AcknowledgeConfig(LM75_I2C, ENABLE);

  // ------------------------------------- Transmission Phase ------------------
  // Send LM75_I2C START condition 
  I2C_GenerateSTART(LM75_I2C, ENABLE);

  // Test on LM75_I2C EV5 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_MODE_SELECT))  /* EV5 */
  {
  }

  // Send STLM75 slave address for write 
  I2C_Send7bitAddress(LM75_I2C, LM75_ADDR, I2C_Direction_Transmitter);

  // Test on LM75_I2C EV6 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED)) /* EV6 */
  {
  }

  // Send the temperature register data pointer 
  I2C_SendData(LM75_I2C, LM75_REG_TEMP);

  // Test on LM75_I2C EV8 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED)) /* EV8 */
  {
  }

  // -------------------------------- Reception Phase --------------------------
  // Send Re-STRAT condition 
  I2C_GenerateSTART(LM75_I2C, ENABLE);

  // Test on EV5 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_MODE_SELECT))  /* EV5 */
  {
  }

  // Send STLM75 slave address for read 
  I2C_Send7bitAddress(LM75_I2C, LM75_ADDR, I2C_Direction_Receiver);

  // Test on EV6 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED))  /* EV6 */
  {
  }

  // Test on EV7 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_BYTE_RECEIVED))  /* EV7 */
  {
  }

  // Store LM75_I2C received data 
  RegValue = I2C_ReceiveData(LM75_I2C) << 8;

  // Disable LM75_I2C acknowledgement 
  I2C_AcknowledgeConfig(LM75_I2C, DISABLE);

  // Send LM75_I2C STOP Condition 
  I2C_GenerateSTOP(LM75_I2C, ENABLE);

  // Test on RXNE flag 
  while (I2C_GetFlagStatus(LM75_I2C, I2C_FLAG_RXNE) == RESET);

  // Store LM75_I2C received data 
  RegValue |= I2C_ReceiveData(LM75_I2C);

  // Return Temperature value 
  return (RegValue >> 7);
}

/**
  * @brief  Read the configuration register from the LM75.
  * @param  None
  * @retval None
  */
uint8_t I2C_LM75_ConfReg_Read(void)
{
  __IO uint8_t RegValue = 0;

  // Enable LM75_I2C acknowledgement if it is already disabled by other function 
  I2C_AcknowledgeConfig(LM75_I2C, ENABLE);
  // ----------------------------- Transmission Phase --------------------------
  // Send LM75_I2C START condition 
  I2C_GenerateSTART(LM75_I2C, ENABLE);

  // Test on LM75_I2C EV5 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_MODE_SELECT))  /* EV5 */
  {
  }

  // Send STLM75 slave address for write 
  I2C_Send7bitAddress(LM75_I2C, LM75_ADDR, I2C_Direction_Transmitter);

  // Test on LM75_I2C EV6 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED)) /* EV6 */
  {
  }

  // Send the configuration register data pointer 
  I2C_SendData(LM75_I2C, LM75_REG_CONF);

  // Test on LM75_I2C EV8 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED)) /* EV8 */
  {
  }

  // ----------------------------- Reception Phase -----------------------------
  // Send Re-STRAT condition 
  I2C_GenerateSTART(LM75_I2C, ENABLE);

  // Test on EV5 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_MODE_SELECT))  /* EV5 */
  {
  }

  // Send STLM75 slave address for read 
  I2C_Send7bitAddress(LM75_I2C, LM75_ADDR, I2C_Direction_Receiver);

  // Test on EV6 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED))  /* EV6 */
  {
  }

  // Disable LM75_I2C acknowledgement 
  I2C_AcknowledgeConfig(LM75_I2C, DISABLE);

  // Send LM75_I2C STOP Condition 
  I2C_GenerateSTOP(LM75_I2C, ENABLE);

  // Test on RXNE flag 
  while (I2C_GetFlagStatus(LM75_I2C, I2C_FLAG_RXNE) == RESET);

  // Store LM75_I2C received data 
  RegValue = I2C_ReceiveData(LM75_I2C);

  // Return configuration register value 
  return (RegValue);
}

/**
  * @brief  Write to the configuration register of the LM75.
  * @param  None
  * @retval None
  */
void I2C_LM75_ConfReg_Write(uint8_t RegValue)
{
  // -------------------------------- Transmission Phase -----------------------
  // Send LM75_I2C START condition 
  I2C_GenerateSTART(LM75_I2C, ENABLE);

  // Test on LM75_I2C EV5 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_MODE_SELECT))  /* EV5 */
  {
  }

  // Send STLM75 slave address for write 
  I2C_Send7bitAddress(LM75_I2C, LM75_ADDR, I2C_Direction_Transmitter);

  // Test on LM75_I2C EV6 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED)) /* EV6 */
  {
  }

  // Send the configuration register data pointer 
  I2C_SendData(LM75_I2C, LM75_REG_CONF);

  // Test on LM75_I2C EV8 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED)) /* EV8 */
  {
  }

  // Send LM75_I2C data 
  I2C_SendData(LM75_I2C, RegValue);

  // Test on LM75_I2C EV8 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED)) /* EV8 */
  {
  }

  // Send LM75_I2C STOP Condition 
  I2C_GenerateSTOP(LM75_I2C, ENABLE);
}

/**
  * @brief  Enables or disables the LM75.
  * @param  None
  * @retval None
  */
void I2C_LM75_ShutDown(FunctionalState NewState)
{
  __IO uint8_t RegValue = 0;

  // ---------------------------- Transmission Phase ---------------------------
  // Send LM75_I2C START condition 
  I2C_GenerateSTART(LM75_I2C, ENABLE);

  // Test on LM75_I2C EV5 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_MODE_SELECT))  /* EV5 */
  {
  }

  // Send STLM75 slave address for write 
  I2C_Send7bitAddress(LM75_I2C, LM75_ADDR, I2C_Direction_Transmitter);

  // Test on LM75_I2C EV6 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED)) /* EV6 */
  {
  }

  // Send the configuration register data pointer 
  I2C_SendData(LM75_I2C, LM75_REG_CONF);

  // Test on LM75_I2C EV8 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED))  /* EV8 */
  {
  }

  // -------------------------------- Reception Phase --------------------------
  // Send Re-STRAT condition 
  I2C_GenerateSTART(LM75_I2C, ENABLE);

  // Test on EV5 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_MODE_SELECT))  /* EV5 */
  {
  }

  // Send STLM75 slave address for read 
  I2C_Send7bitAddress(LM75_I2C, LM75_ADDR, I2C_Direction_Receiver);

  // Test on EV6 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED))  /* EV6 */
  {
  }

  // Disable LM75_I2C acknowledgement 
  I2C_AcknowledgeConfig(LM75_I2C, DISABLE);

  // Send LM75_I2C STOP Condition 
  I2C_GenerateSTOP(LM75_I2C, ENABLE);

  // Test on RXNE flag 
  while (I2C_GetFlagStatus(LM75_I2C, I2C_FLAG_RXNE) == RESET);

  // Store LM75_I2C received data 
  RegValue = I2C_ReceiveData(LM75_I2C);

  // ------------------------------------ Transmission Phase -------------------
  // Send LM75_I2C START condition 
  I2C_GenerateSTART(LM75_I2C, ENABLE);

  // Test on LM75_I2C EV5 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_MODE_SELECT))  /* EV5 */
  {
  }

  // Send STLM75 slave address for write 
  I2C_Send7bitAddress(LM75_I2C, LM75_ADDR, I2C_Direction_Transmitter);

  // Test on LM75_I2C EV6 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED)) /* EV6 */
  {
  }

  // Send the configuration register data pointer 
  I2C_SendData(LM75_I2C, LM75_REG_CONF);

  // Test on LM75_I2C EV8 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED)) /* EV8 */
  {
  }

  // Enable or disable SD bit 
  if (NewState != DISABLE)
  {
    // Enable LM75 
    I2C_SendData(LM75_I2C, RegValue & LM75_SD_RESET);
  }
  else
  {
    // Disable LM75 
    I2C_SendData(LM75_I2C, RegValue | LM75_SD_SET);
  }

  // Test on LM75_I2C EV8 and clear it 
  while (!I2C_CheckEvent(LM75_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED)) /* EV8 */
  {
  }

  // Send LM75_I2C STOP Condition 
  I2C_GenerateSTOP(LM75_I2C, ENABLE);

}

